/*

	BOINC Stats Windows Gadget Version 0.3

	This widget uses the data held by http://boinc.mundayweb.com to display user's stats
	on their desktops.

	A working Internet connection is required by the widget.

	sprintf function provided by http://www.webtoolkit.info 
	
	Copyright (C) 2011  Neil Munday (neil@mundayweb.com)

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

	A full copy of the terms of the GPL in the included text file "gpl.txt".

	------------------------------------------------------------------------------------
	Revision history
	------------------------------------------------------------------------------------

	Date		Version		Comments

	21/12/11	0.1			First release.

	22/12/11	0.2			Bug fix for JavaScript's handling of floats.
							The gadget will now detect when a new version is available.

	31/12/11	0.3			Ability to select how many projects to show added.

	01/01/12	0.4			Condensed row spacing.

	18/02/14	0.5			Reverted to basic style and now allows colour customisation.

	05/04/14	0.6			Added rank column and adjusted text alignment.

	06/04/14	0.7			Fixed alignment of total columns and colour of footer text.

	11/03/14	0.8			Credit totals and RAC values are now display as comma separated values.

	25/03/14	0.9			Added setting to allow decimal places to be toggled on/off.

	05/01/17	1.0			Changed feed URL to go direct to HTTPS rather than via CloudFlare's page redirect which was causing problems.
							Thanks to Gareth Lock for the heads up and for providing a fix.
*/

var url = 'https://boinc.mundayweb.com/xml/feed.php?userID=';
var timeTransition = 2;
var dockedHeight = 75;
var dockedWidth = 130;
var undockedHeight = 306;
var undockedWidth = 399;
var rowHeight = 20;
var headerHeight = 37;
var footerHeight = headerHeight;
var version = 1.0;
var latestVersion = version;
var projects = null;
var totalCredit = 0;
var totalRAC = 0;
var showProjects = 0;
var statsDate = '';
var name = '';
var rankFormat = 0;
var errorMsg = '';
var gadget;
var refreshInterval = 3600 * 2 * 1000; // milliseconds - please do not modify!
var timerStarted = false;
var timer;
var initialLoad = true;
var txtColour = defaultTxtColour;
var showDp = defaultDp;

function init(){
	initialLoad = true;
	gadget = document.getElementById('gadget');

	System.Gadget.settingsUI = "settings.html";
	System.Gadget.onSettingsClosed = update;
	System.Gadget.onDock = checkDockState;
	System.Gadget.onUndock = checkDockState;

	update();
}

function insertCommas(x) {
    return x.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
}

function update(){
	var bgColour = System.Gadget.Settings.read("bgColour");
	if (bgColour == ''){
		bgColour = defaultBgColour;
	}

	var borderColour = System.Gadget.Settings.read("borderColour");
	if (borderColour == ''){
		borderColour = defaultBorderColour;
	}
	gadget.style.borderColor = borderColour;

	var showBorder = System.Gadget.Settings.read("showBorder");
	if (showBorder === ""){
		showBorder = defaultShowBorder;
	}

	if (!showBorder){
		gadget.style.borderStyle = 'none';
	}
	else{
		gadget.style.borderStyle = 'solid';
	}

	showDp = System.Gadget.Settings.read("showDp");

	gadget.style.backgroundColor = bgColour;

	txtColour = System.Gadget.Settings.read("txtColour");
	if (txtColour == ''){
		txtColour = defaultTxtColour;
	}

	var x = System.Gadget.Settings.read("showProjects");
	var origValue = showProjects;

	if (x == ""){
		showProjects = 0;
	}
	else{
		showProjects = x;
	}

	if (!System.Gadget.docked && showProjects != origValue){
		checkDockState();
	}

	var userId = System.Gadget.Settings.read("userId");
	if (userId == null || userId == '' || userId == 0){
		gadget.innerHTML = 'Please enter a user ID';
	}
	else{
		getXML(userId); // asynchronous function!
	}
}

function drawDocked(){
	gadget.style.padding = '0px';

	if (projects == null || projects.length == 0){
		gadget.innerHTML = "<div style=\"width: " + dockedWidth + "; height: " + dockedHeigth + ";\">Please enter a user ID</div>";
		return;
	}

	var str = '<table class="statsTable">';

	str += '<tr>';
	str += '<td class="dockedTitle" colspan="3"><span style="color: ' + txtColour + ';">BOINC</span></td>';
	str += '</tr>';
	str += '<tr>';
	str += '<td class="center" colspan="3"><span style="color: ' + txtColour + ';">' + formatNumber(totalCredit) + '</span></td>';
	str += '</tr>';
	str += '<tr>';
	str += '<td class="center" colspan="3"><span style="color: ' + txtColour + ';">' + formatNumber(totalRAC) + '</span></td>';
	str += '</tr>';

	str += '<tr>';
	str += '<td width="33%">&nbsp;</td>';
	str += '<td width="34%" class="footerCenter"><span style="color: ' + txtColour + ';">' + statsDate + '</span></td>';
	str += '<td width="33%" class="footerRight"><span style="color: ' + txtColour + ';">';
	if (version == latestVersion){
		str += version;
	}
	else{
		str += '<a class="newVersion" href="http://boinc.mundayweb.com/desktop/windows.php" title="A new version is available for download">' + version + '</a>';
	}
	str += '</span></td>';
	str += '</tr>';
	str += '</table>';

	gadget.innerHTML = str;
}

function drawUndocked(){
	gadget.style.padding = '10px';

	if (projects == null || projects.length == 0){
		gadget.innerHTML = "<div style=\"width: " + undockedWidth + "; height: " + undockedHeigth + ";\">Please enter a user ID</div>";
		return;
	}

	var rankStr = '';
	if (rankFormat == 1){
		rankStr = '%';
	}

	var str = '<table class="statsTable" border="0">';

	str += '<tr">';
	str += '<td colspan="3" class="center bold"><span style="color: ' + txtColour + ';">' + name + '</span></td>';
	str += '</tr>';

	var limit = projects.length;
	if (showProjects > 0 && showProjects <= projects.length){
		limit = showProjects;
	}

	for (var i = 0; i < limit; i++){
		str += '<tr>';
		str += '<td><span style="color: ' + txtColour + ';">' + projects[i].name + '</span></td>';
		str += '<td align="right"><span style="color: ' + txtColour + ';">' + formatNumber(projects[i].credit) + '</span></td>';
		str += '<td align="right"><span style="color: ' + txtColour + ';">' + formatNumber(projects[i].rac) + '</span></td>';
		str += '<td align="right"><span style="color: ' + txtColour + ';">(' + projects[i].rank + rankStr + ')</span></td>';
		str += '</tr>';
	}

	var otherCredit = 0;
	var otherRAC = 0;

	if (showProjects > 0 && projects.length > showProjects){
		for (var i = showProjects; i < projects.length; i++){
			otherCredit += projects[i].credit;
			otherRAC += projects[i].rac;
		}

		str += '<tr>';
		str += '<td><span style="color: ' + txtColour + ';">Others (' + (projects.length - showProjects) + '):</span></td>';
		str += '<td align="right"><span style="color: ' + txtColour + ';">' + formatNumber(otherCredit) + '</span></td>';
		str += '<td align="right"><span style="color: ' + txtColour + ';">' + formatNumber(otherRAC) + '</span></td>';
		str += '<td align="right">&nbsp;</td>';
		str += '</tr>';
	}

	str += '<tr>';
	str += '<td><span style="color: ' + txtColour + ';">Total:</span></td>';
	str += '<td align="right"><span style="color: ' + txtColour + ';">' + formatNumber(totalCredit) + '</span></td>';
	str += '<td align="right"><span style="color: ' + txtColour + ';">' + formatNumber(totalRAC) + '</span></td>';
	str += '<td align="right">&nbsp;</td>';
	str += '</tr>';

	str += '<tr class="footer">';
	str += '<td colspan="4">';
	str += '<table class="statsTable">';
	str += '<tr>';
	str += '<td class="footerLeft"><span style="color: ' + txtColour + ';">' + statsDate + '</span></td>';
	str += '<td class="footerCenter"><span style="color: ' + txtColour + ';"><a class="footer" style="color: ' + txtColour + ' !important;" href="http://boinc.mundayweb.com">http://boinc.mundayweb.com</a></span></td>';
	str += '<td class="footerRight"><span style="color: ' + txtColour + ';">';
	if (version == latestVersion){
		str += version;
	}
	else{
		str += '<a class="newVersion" href="http://boinc.mundayweb.com/desktop/windows.php" title="A new version is available for download">' + version + '</a>';
	}
	str += '</span></td>';
	str += '</tr>';
	str += '</table>';
	str += '</td>';
	str += '</tr>';

	str += '</table>';

	gadget.innerHTML = str;
}

function formatNumber(number){
	if (showDp){
		return insertCommas(sprintf("%01.2f", number));
	}
	return insertCommas(sprintf("%d", number));
}

function getXML(userId) {
	var timestamp = new Date().getTime();
    var xobj = new ActiveXObject("Msxml2.XMLHTTP");
    xobj.open("GET", url + userId + '&ts=' + timestamp, true);
    xobj.onreadystatechange = function(){
        if (xobj.readyState == 4){
            if (xobj.status == 200 || xobj.status == 304){
                var xml = xobj.responseXML;
				parseFeed(xml);
				if (initialLoad){
					initialLoad = false;
					checkDockState();
				}
				// now draw stats
				if (System.Gadget.docked){
					drawDocked();
				}
				else{
					drawUndocked();
				}
				timer = setTimeout(update, refreshInterval);
			}
        }
		else{
			//gadget.innerHTML = "connecting...";
        }
    }
    xobj.send();
}

function parseFeed(xml){
	projects = new Array();
	var userXML = xml.getElementsByTagName('user');
	if (userXML.length > 0){
		totalCredit = userXML[0].getAttribute('credit');
		totalRAC = userXML[0].getAttribute('rac');
		statsDate = userXML[0].getAttribute('updated');
		name = userXML[0].getAttribute('name');
		rankFormat = userXML[0].getAttribute('rankFormat');
	}
	else{
		gadget.innerHTML = 'Invalid User ID!';
		return;
	}

	var projectXML = xml.getElementsByTagName('project');
	for (var i = 0; i < projectXML.length; i++){
		projects[i] = {
			'name': projectXML[i].getAttribute('name'),
			'credit': parseFloat(projectXML[i].getAttribute('credit')),
			'rac': parseFloat(projectXML[i].getAttribute('rac')),
			'rank': parseFloat(projectXML[i].getAttribute('rank'))
		};
	}

	var gadgetXML = xml.getElementsByTagName('windows');
	if (gadgetXML.length > 0){
		latestVersion = gadgetXML[0].getAttribute('version');
	}
}

function checkDockState(){
	//var bgImg = document.getElementById("bgImg");
			
    // Set the width of the background element to 0.
    // This forces the image to be refreshed appropriately.
    //bgImg.style.width = 0;

    System.Gadget.beginTransition();
    
    var oBody = document.body.style;
    if (System.Gadget.docked){
        oBody.width = dockedWidth;
        oBody.height = dockedHeight;
		gadget.style.width = dockedWidth;
		gadget.style.height = dockedHeight;
		drawDocked();
    }
    else{
		if (projects && projects.length > 0){
			var count = projects.length;
			if (showProjects > 0 && showProjects <= projects.length){
				count = showProjects + 1;
			}
			undockedHeight = (rowHeight * (count + 1)) + headerHeight + footerHeight + 10;
		}

        oBody.width = undockedWidth;
        oBody.height = undockedHeight;
		gadget.style.width = undockedWidth;
		gadget.style.height = undockedHeight;
		drawUndocked();
    }
    System.Gadget.endTransition(System.Gadget.TransitionType.morph, timeTransition);
}
